import Tool from "./tool";
import { graph, state, renderer } from "../editor";
import { CONTEXT } from "../state";
import { CollectMenu, COLLECTION_KEY } from "./menus/collectmenu";
import * as Graph from "../graph";
import jquery from "jquery";

/**
 * Only one instance of this should exist, since box-select has to work on a global scale.
 */
var collectToolInstance = undefined; // Used for box select
export default class CollectTool extends Tool {
    constructor(key) {
        super("Collect", "collect", key, new CollectMenu());
        this.setupBoxSelect();

        if (collectToolInstance === undefined) {
            collectToolInstance = this;
        }
    }

    onBoxSelect(left, bottom, top, right) {
        // Filter out selected nodes
        const hitNodes = [];
        const tl = renderer.screen2GraphCoords(left, top);
        const br = renderer.screen2GraphCoords(right, bottom);
        graph.data[Graph.GRAPH_NODES].forEach((node) => {
            if (
                tl.x < node.x &&
                node.x < br.x &&
                br.y > node.y &&
                node.y > tl.y
            ) {
                hitNodes.push(node);
            }
        });

        // Add to selected items
        if (state.itemsContext !== CONTEXT.node) {
            state.clearSelectedItems();
            state.itemsContext = CONTEXT.node;
        }
        state.addSelectedItems(hitNodes);
        this.menu.value(COLLECTION_KEY, state.selectedItems);
    }

    onNodeClick(node) {
        if (state.itemsContext !== CONTEXT.node) {
            state.clearSelectedItems();
            state.itemsContext = CONTEXT.node;
        }

        if (state.selectedItems.has(node)) {
            state.removeSelectedItem(node);
        } else {
            state.addSelectedItem(node);
        }

        this.menu.value(COLLECTION_KEY, state.selectedItems);
    }

    onLinkClick(link) {
        if (state.itemsContext !== CONTEXT.link) {
            state.clearSelectedItems();
            state.itemsContext = CONTEXT.link;
        }

        if (state.selectedItems.has(link)) {
            state.removeSelectedItem(link);
        } else {
            state.addSelectedItem(link);
        }

        this.menu.value(COLLECTION_KEY, state.selectedItems);
    }

    onMenuChange(key, value) {
        if (key === COLLECTION_KEY && value === undefined) {
            state.clearSelectedItems();
            this.menu.value(COLLECTION_KEY, []);
        }
    }

    setupBoxSelect() {
        window.addEventListener("load", () => {
            // Source: https://github.com/vasturiano/force-graph/issues/151#issuecomment-735850938

            // forceGraph element is the element provided to the Force Graph Library
            jquery("#2d-graph").on(
                "pointerdown",
                this.boxSelectOnPointerDown
            );
            jquery("#2d-graph").on(
                "pointermove",
                this.boxSelectOnPointerMove
            );
            jquery("#2d-graph").on(
                "pointerup",
                this.boxSelectOnPointerUp
            );
        });
    }

    boxSelectOnPointerDown(e) {
        if (!e.shiftKey) {
            return;
        }

        e.preventDefault();
        this.boxSelect = document.createElement("div");
        this.boxSelect.id = "box-select";
        this.boxSelect.style.left = e.offsetX.toString() + "px";
        this.boxSelect.style.top = e.offsetY.toString() + "px";
        this.boxSelectStart = {
            x: e.offsetX,
            y: e.offsetY,
        };
        // app element is the element just above the forceGraph element.
        jquery("#box-select-layer").append(this.boxSelect);
    }

    boxSelectOnPointerMove(e) {
        if (!this.boxSelect) {
            return;
        }

        if (!e.shiftKey) {
            this.boxSelect.remove();
            return;
        }

        e.preventDefault();
        if (e.offsetX < this.boxSelectStart.x) {
            this.boxSelect.style.left = e.offsetX.toString() + "px";
            this.boxSelect.style.width =
                (this.boxSelectStart.x - e.offsetX).toString() + "px";
        } else {
            this.boxSelect.style.left = this.boxSelectStart.x.toString() + "px";
            this.boxSelect.style.width =
                (e.offsetX - this.boxSelectStart.x).toString() + "px";
        }
        if (e.offsetY < this.boxSelectStart.y) {
            this.boxSelect.style.top = e.offsetY.toString() + "px";
            this.boxSelect.style.height =
                (this.boxSelectStart.y - e.offsetY).toString() + "px";
        } else {
            this.boxSelect.style.top = this.boxSelectStart.y.toString() + "px";
            this.boxSelect.style.height =
                (e.offsetY - this.boxSelectStart.y).toString() + "px";
        }
    }

    boxSelectOnPointerUp(e) {
        if (!this.boxSelect) {
            return;
        }

        if (!e.shiftKey) {
            this.boxSelect.remove();
            return;
        }

        e.preventDefault();
        let left, bottom, top, right;
        if (e.offsetX < this.boxSelectStart.x) {
            left = e.offsetX;
            right = this.boxSelectStart.x;
        } else {
            left = this.boxSelectStart.x;
            right = e.offsetX;
        }
        if (e.offsetY < this.boxSelectStart.y) {
            top = e.offsetY;
            bottom = this.boxSelectStart.y;
        } else {
            top = this.boxSelectStart.y;
            bottom = e.offsetY;
        }
        this.boxSelect.remove();
        collectToolInstance.onBoxSelect(left, bottom, top, right);
    }
}
